<?php
public function getSitePath()
{
    return $this->config->site->id;
}

public function setSavePath($objectType = '')
{
    $sitePath = $this->getSitePath();
    if($objectType == 'source')
    {
        $template = $this->config->template->{$this->app->clientDevice}->name;
        $theme    = $this->config->template->{$this->app->clientDevice}->theme;
        $savePath = $this->app->getDataRoot() . "source/{$sitePath}/{$template}/{$theme}/";
        $this->savePath = $this->app->getDataRoot();
    }
    elseif($objectType == 'slide')
    {
        $theme    = $this->config->template->{$this->app->clientDevice}->theme;
        $savePath = $this->app->getDataRoot() . "slides/{$sitePath}/{$theme}/";
        $this->savePath = $this->app->getDataRoot();
    }
    elseif($objectType == 'themePackage')
    {
        $savePath = $this->app->getTmpRoot() . "package/{$sitePath}/";
        $this->savePath = $this->app->getTmpRoot();
    }
    else
    {
        $savePath = $this->app->getDataRoot() . "upload/{$sitePath}/" . date('Ym/', $this->now);
        $this->savePath = dirname($savePath) . '/';
    } 

    if(!file_exists($savePath))
    {
        @mkdir($savePath, 0777, true);
        if(is_writable($savePath) && !file_exists($savePath . DS . 'index.php'))
        {
            $fd = @fopen($savePath . DS . 'index.php', "a+");
            fclose($fd);
            chmod($savePath . DS . 'index.php' , 0755);
        }
    }
}

public function setPathName($file, $objectType = 'upload')
{
    if(strpos('slide,source,themePackage', $objectType) === false)
    {
        $sessionID  = session_id();
        $randString = substr($sessionID, mt_rand(0, strlen($sessionID) - 5), 3);
        $pathName   = date('Ym/dHis', $this->now) . $file['id'] . mt_rand(0, 10000) . $randString;
    }
    elseif($objectType == 'source') 
    {
        /* Process file path if objectType is source. */
        $template = $this->config->template->{$this->app->clientDevice}->name;
        $theme    = $this->config->template->{$this->app->clientDevice}->theme;
        $sitePath = $this->getSitePath();
        return "source/{$sitePath}{$template}/{$theme}/{$file['title']}.{$file['extension']}";
    }
    elseif($objectType == 'themePackage')
    {
        return "{$file['title']}.{$file['extension']}"; 
    }
    
    /* rand file name more */
    list($path, $fileName) = explode('/', $pathName);
    $fileName = md5(mt_rand(0, 10000) . str_shuffle(md5($fileName)) . mt_rand(0, 10000));
    return $path . '/f_' . $fileName . '.' . $file['extension'];
}



public function setWebPath()
{
    $sitePath = $this->getSitePath();
    $this->webPath = $this->app->getWebRoot() . "data/upload/{$sitePath}/";
}

public function getWebPath($objectType = '')
{
    if(strpos(',slide,source,', ",$objectType,") !== false) return $this->app->getWebRoot() . "data/";
    return $this->webPath;
}

public function canUpload()
{
    /* check usage. */
    if(isset($this->config->global->usage) and $this->config->global->usage > $this->config->zhanqun->plan->{$this->config->service->plan}->usage) return false;
    return parent::canUpload();
}

public function scanSources($template, $theme)
{
    $sitePath = $this->getSitePath();
    $fileList = glob($this->app->getDataRoot() . "source/{$sitePath}/$template/$theme/*");
    $newFiles = array();

    foreach($fileList as $file)
    {
        $info = pathinfo($file);
        if(!in_array($info['extension'], $this->config->file->imageExtensions)) continue;
        $file = str_replace($this->app->getDataRoot(), '', $file);
        $filesCount = $this->dao->select('count(*) as count')->from(TABLE_FILE)->where('pathname')->eq($file)->fetch('count');
        if($filesCount) continue;
        $newFiles[] = $file;
    }

    foreach($newFiles as $path)
    {
        $info = pathinfo($path);
        $file = new stdclass();
        $file->pathname   = $path;
        $file->title      = $info['filename'];
        $file->lang       = 'all';
        $file->objectType = 'source';
        $file->objectID   = "{$template}_{$theme}";
        $file->addedDate  = helper::now();
        $file->extension  = $info['extension'];
        $this->dao->insert(TABLE_FILE)->data($file)->exec();
    }

    return true;
}

public function delete($fileID, $null = null)
{
    $file = $this->getByID($fileID);
    if(file_exists($file->realPath)) unlink($file->realPath);
    if(in_array($file->extension, $this->config->file->imageExtensions)); 
    {
        foreach($this->config->file->thumbs as $size => $configure)
        {
            $thumbPath = $this->app->getDataRoot() . "upload/" . $this->getSitePath() . '/' . str_replace('f_', $size . '_', $file->pathname);
            if(file_exists($thumbPath)) unlink($thumbPath); 
        }
    }

    $this->dao->delete()->from(TABLE_FILE)
        ->where('id')->eq($file->id)
        ->beginIf(RUN_MODE == 'front' and $this->app->user->admin == 'no')->andWhere('addedBy')->eq($this->app->user->account)->fi()
        ->exec();

    return !dao::isError();
}

public function getRealPathName($pathName, $objectType = '')
{
    $sitePath = $this->getSitePath();
    $realPath = $this->app->getDataRoot() . "upload/{$sitePath}/{$pathName}";
    if($objectType == 'source' or $objectType == 'slide') $realPath = $this->app->getDataRoot() . $pathName;

    if(file_exists($realPath)) return $pathName;

    return $this->getSaveName($pathName);
}

/**
 * Get the list of invalid files
 * 
 * @param  void
 * @access public
 * @return array
 */

public function getInvalidList($pager = null)
{
    $dataRoot     = $this->app->getDataRoot();
    $sitePath     = $this->getSitePath();
    $uploadFiles  = glob($dataRoot . "/upload/{$sitePath}/*/*"); 
    $invalidFiles = array();
    foreach($uploadFiles as $uploadFile)
    {
        $name = basename($uploadFile);
        $tag  = substr(substr($name, strpos($name, '_') + 1), 0, strpos($name, '.') - 2);
        if(strlen($tag) < 10) continue;
        if(!$this->checkExistence($tag)) $invalidFiles[] = $uploadFile; 
    }
    
    $unusedFiles = array();
    foreach($invalidFiles as $invalidFile) 
    { 
        $unusedFile               = new stdclass();
        $unusedFile->realPathname = $invalidFile; 
        $unusedFile->pathname     = substr($invalidFile, strlen($this->app->getDataRoot() . 'upload/'));
        $unusedFile->extension    = $this->getExtension($invalidFile);
        $unusedFile->size         = filesize($invalidFile);
        $unusedFile->addedDate    = date("Y-m-d H:i:s", filemtime($invalidFile));
              
        $unusedFiles[] = $unusedFile;
    }   
    if($pager !== null)
    {
        $pager->recTotal  = count($unusedFiles);
        $pager->pageTotal = ceil($pager->recTotal / $pager->recPerPage);
        $unusedFiles      = array_slice($unusedFiles, ($pager->pageID - 1) * $pager->recPerPage, $pager->recPerPage);
    }

    return $unusedFiles;
}
